/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef headerfile_ezc_cache
#define headerfile_ezc_cache

#include "item.h"
#include "functions.h"



namespace Ezc
{

class Blocks;


template<class StreamType>
void Cache(Functions<StreamType> & fun, Item::Function & function)
{
	function.fun_cache = 0;

	if( !function.name.empty() && function.arg < 0 )
	{
		typename Functions<StreamType>::Iterator i = fun.Find(function.name);

		if( i != fun.End() )
			function.fun_cache = &i->second;
	}

	for(size_t i=0 ; i < function.parameters.size() ; ++i)
		Cache(fun, *function.parameters[i]);
}


template<class StreamType>
void Cache(Functions<StreamType> & fun, Item & item)
{
	// one exception (if_index is putting its argument on the functions stack)
	/*
		!! IMPROVE ME we need to change the parser and the index take as a string
	 */
	if( item.type != Item::item_ifindex )
	{
		for(size_t f=0; f < item.functions.size() ; ++f)
			Cache(fun, item.functions[f]);
	}

	for(size_t i=0; i < item.item_tab.size() ; ++i)
		Cache(fun, *item.item_tab[i]);
}



void Cache(Blocks & blocks, Item & item);



} // namespace Ezc



#endif
