/*
 * This file is a part of EZC -- Easy templating in C++ library
 * and is distributed under the BSD 3-Clause licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/* 
 * Copyright (c) 2007-2014, Tomasz Sowa
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *    
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *    
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef headerfile_ezc_item
#define headerfile_ezc_item

#include <vector>
#include <string>
#include "funinfo.h"
#include "functions.h"


namespace Ezc
{



struct Item
{
	enum Type 
	{
		item_none, item_container, item_text, item_normal, item_is, item_isno,
		item_if, item_ifno, item_ifany, item_ifone, item_ifanyno, item_ifoneno, item_ifindex,
		item_for, item_else, item_end, item_err, item_include, item_comment, item_def, item_filter,
		item_ezc, item_block
	};


	struct Function
	{
		std::wstring name;				// function name (can be empty, means there is no any function -- only a text parameter)
		std::wstring par;				// parameter
		std::vector<Function*> parameters;
		void * fun_cache; 				// only valid if name is not empty
		Item * item_block;				
		int arg;						// used if name is numeric (if no then is equal -1)


		Function()
		{
			fun_cache  = 0;
			item_block = 0;
			arg        = -1;
		}

		~Function()
		{
			Clear();
		}

		Function(const Function & f)
		{
			operator=(f);
		}

		Function & operator=(const Function & f)
		{
			name       = f.name;
			par        = f.par;
			fun_cache  = f.fun_cache;
			item_block = f.item_block;
			arg        = f.arg;

			for(size_t i=0 ; i<f.parameters.size() ; ++i)
				parameters.push_back(new Function(*f.parameters[i]));

		return *this;
		}

		Function & AddNewParam()
		{
			parameters.push_back(new Function());

		return *parameters.back();
		}

		void Clear()
		{
			for(size_t i=0 ; i<parameters.size() ; ++i)
				delete parameters[i];

			parameters.clear();
			name.clear();
			par.clear();
			fun_cache  = 0;
			item_block = 0;
			arg        = -1;
		}

	};



	Type type;
	std::wstring text;						// used in: item_text
	std::wstring file_name;					// used in: item_include (as a file name)
	std::vector<Item*> item_tab;			// childs
	std::vector<Function> functions;

	Item();
	Item(const Item & i);
	Item & operator=(const Item & i);
	void CopyItemTable(const Item & i);
	~Item();

	Item * AddItem(const Item * porg = 0);
	Item * AddItem(const Item & porg);
	void   ClearItems();
	Type   LastItemType();
	void   DeleteLastItem();
	void   Clear();
	void   ClearCache();

};




} // namespace Ezc


#endif
